/* eslint-disable no-magic-numbers */
import { store } from '@/redux/store';
import { configurationFixture } from '@/models/fixtures/configurationFixture';
import { configurationMapper } from './pluginsManager.utils';
import { PluginsManager } from './pluginsManager';

jest.mock('../../redux/store');

describe('PluginsManager', () => {
  const originalWindow = { ...global.window };

  beforeEach(() => {
    global.window = { ...originalWindow };
  });

  afterEach(() => {
    global.window = originalWindow;
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('setHashedPlugin correctly computes hash and updates hashedPlugins', () => {
    const pluginUrl = 'https://example.com/plugin.js';
    const pluginScript = 'console.log("Hello, Plugin!");';

    PluginsManager.setHashedPlugin({ pluginUrl, pluginScript });

    expect(PluginsManager.hashedPlugins[pluginUrl]).toBe('edc7eeafccc9e1ab66f713298425947b');
  });

  it('init subscribes to store changes and updates minerva configuration', () => {
    (store.getState as jest.Mock).mockReturnValueOnce({
      configuration: { main: { data: configurationFixture } },
    });

    PluginsManager.init();

    expect(store.subscribe).toHaveBeenCalled();

    // Simulate store change
    (store.subscribe as jest.Mock).mock.calls[0][0]();

    expect(store.getState).toHaveBeenCalled();
    expect(window.minerva.configuration).toEqual(configurationMapper(configurationFixture));
  });
  it('init does not update minerva configuration when configuration is undefined', () => {
    (store.getState as jest.Mock).mockReturnValueOnce({
      configuration: { main: { data: undefined } },
    });

    PluginsManager.init();

    expect(store.subscribe).toHaveBeenCalled();

    // Simulate store change
    (store.subscribe as jest.Mock).mock.calls[0][0]();

    expect(store.getState).toHaveBeenCalled();
    expect(window.minerva.configuration).toBeUndefined();
  });

  it('registerPlugin dispatches action and returns element', () => {
    const pluginName = 'TestPlugin';
    const pluginVersion = '1.0.0';
    const pluginUrl = 'https://example.com/test-plugin.js';

    const result = PluginsManager.registerPlugin({ pluginName, pluginVersion, pluginUrl });

    expect(store.dispatch).toHaveBeenCalled();

    expect(result.element).toBeDefined();
  });
});
