import md5 from 'crypto-js/md5';
import { store } from '@/redux/store';
import { registerPlugin } from '@/redux/plugins/plugins.thunks';
import { configurationMapper } from './pluginsManager.utils';
import type { PluginsManagerType } from './pluginsManager.types';

export const PluginsManager: PluginsManagerType = {
  hashedPlugins: {},
  setHashedPlugin({ pluginUrl, pluginScript }) {
    const hash = md5(pluginScript).toString();

    PluginsManager.hashedPlugins[pluginUrl] = hash;

    return hash;
  },
  init() {
    window.minerva = {
      plugins: {
        registerPlugin: PluginsManager.registerPlugin,
      },
    };

    const unsubscribe = store.subscribe(() => {
      const configurationStore = store.getState().configuration.main.data;

      if (configurationStore) {
        const configuration = configurationMapper(configurationStore);

        window.minerva = {
          ...window.minerva,
          configuration,
        };
      }
    });

    return unsubscribe;
  },

  registerPlugin({ pluginName, pluginVersion, pluginUrl }) {
    const hash = PluginsManager.hashedPlugins[pluginUrl];

    store.dispatch(
      registerPlugin({
        hash,
        isPublic: false,
        pluginName,
        pluginUrl,
        pluginVersion,
      }),
    );

    // TODO: replace when plugins drawer is implemented
    const element = document.createElement('div');
    const wrapper = document.querySelector('#plugins');
    wrapper?.append(element);

    return {
      element,
    };
  },
};
